
// SPON CrewNotify v0.1.0 (Addon version)
//
// Copyright (C) 2007-9 Bil Bas (bil {dot} bagpuss [at] gmail {dot} com / Spooner)
// License: GNU Lesser General Public License, version 3 <http://www.gnu.org/licenses/>

#ifndef SPON_ADDON
#define SPON_ADDON
#endif

// -----------------------------------------------------------------------------
#define SPON_THIS_FILE SPON\CrewNotify\initCrewNotify
//
// Description:
//   Notifies the player when other players get into, or out of, his vehicle.
//
// Parameters:
//   None.
//
// Returns:
//   nil
// -----------------------------------------------------------------------------

#include "macros\macros.inc.sqf"

#define HANDLERS_VAR "SPON_CN_handlers"
#define GETIN_INDEX 0
#define GETOUT_INDEX 1

#define POLL_DELAY 1.32

#define DEFAULT_SHOW_AI false

// -----------------------------------------------------------------------------

if (isServer) then
{
	SPON_CrewNotify_serverVersion = 'v0.1.0';
	publicVariable "SPON_CrewNotify_serverVersion";
};

// Everything after this point is client-only and pointless in SP.
if (SPON_isDedicatedServer or (not SPON_isMP)) exitWith { nil };

// -----------------------------------------------------------------------------
// Ensure that CrewNotify isn't initialised more than once.
if (not (isNil "SPON_CrewNotify_version")) exitWith
{
	private "_version";
	_version = 'v0.1.0';
	SPON_TRACE_2("SPON CrewNotify run when already initialised",SPON_CrewNotify_version,_version);
	
	nil; // Return.
};

SPON_CrewNotify_version = 'v0.1.0';

// Ensure that isAddon is false if we are running from a mission-script.
#ifdef SPON_ADDON
SPON_CrewNotify_isAddon = true;
#else
SPON_CrewNotify_isAddon = false;
#endif

// -----------------------------------------------------------------------------
// Add getin and getout handlers to the player's vehicle.
SPON_CN_addHandlers =
{
	SPON_GET_PARAMS_1(_vehicle);
	
	_getInIndex = _vehicle addEventHandler ["GETIN",
	{
		SPON_GET_PARAMS_3(_vehicle,_position,_man);
		
		if (((isPlayer _man) or SPON_CN_showAi) and 
			(_vehicle == (vehicle player)) and
			(_man != player) and
			(alive _man)) then
		{
			SPON_CN_system globalChat format [localize "STR_SPON_CN_GETIN",
				[_man] call SPON_CN_manName, [_vehicle] call SPON_CN_vehicleName];
		};
	}];
	
	_getOutIndex = _vehicle addEventHandler ["GETOUT",
	{
		SPON_GET_PARAMS_3(_vehicle,_position,_man);
		
		if (((isPlayer _man) or SPON_CN_showAi) and
			(_vehicle == (vehicle player)) and
			(_man != player) and
			(alive _man)) then
		{
			SPON_CN_system globalChat format [localize "STR_SPON_CN_GETOUT",
				[_man] call SPON_CN_manName, [_vehicle] call SPON_CN_vehicleName];
		};
	}];
	
	_vehicle setVariable [HANDLERS_VAR, [_getInIndex, _getOutIndex]];
	
	SPON_TRACE_2("Added handlers",_vehicle,typeOf _vehicle);
	
	nil; // Return.
};

// -----------------------------------------------------------------------------

SPON_CN_removeHandlers =
{
	SPON_GET_PARAMS_1(_vehicle);
	
	private "_handlers";
	
	_handlers = _vehicle getVariable HANDLERS_VAR;
	
	_vehicle removeEventHandler ["GETOUT", _handlers select GETOUT_INDEX];
	_vehicle removeEventHandler ["GETIN", _handlers select GETIN_INDEX];
	
	SPON_TRACE_2("Removed handlers",_vehicle,typeOf _vehicle);
	
	nil; // Return.
};

// -----------------------------------------------------------------------------
// Get the display name of a particular vehicle.
SPON_CN_vehicleName =
{
	SPON_GET_PARAMS_1(_vehicle);
	
	// Return.
	getText (configFile >> "CfgVehicles" >> typeOf _vehicle >> "displayName");
};

// -----------------------------------------------------------------------------
// Get the display name of a particular man.
SPON_CN_manName =
{
	SPON_GET_PARAMS_1(_man);
	
	private "_id";
	
	_id = [_man] call SPON_CN_groupID;
	
	if ((group _man) != (group player)) then
	{
		_id = format ["%1 %2", group _man, _id];
	};
	
	format ["%1 (%2)", name _man, _id]; // Return.
};

// -----------------------------------------------------------------------------

SPON_CN_groupID = compile preprocessFileLineNumbers "\SPON_CrewNotify\SPON\CrewNotify\groupID.sqf";

SPON_CN_system = "Logic" createVehicleLocal [0, 0];

SPON_CN_showAi = DEFAULT_SHOW_AI;

[] spawn
{
	waitUntil { alive player };

	while { true } do
	{
		while { SPON_ON_FOOT(player) } do
		{
			sleep POLL_DELAY;
		};
		
		_currentVehicle = vehicle player;
		[_currentVehicle] call SPON_CN_addHandlers;
		
		while { (vehicle player) == _currentVehicle } do
		{
			 sleep POLL_DELAY;
		};
		
		[_currentVehicle] call SPON_CN_removeHandlers;
	};
};

nil; // Return.
