// SPON CrewNotify v0.1.0 (Mission-script version)
//
// Copyright (C) 2007-9 Bil Bas (bil {dot} bagpuss [at] gmail {dot} com / Spooner)
// License: GNU Lesser General Public License, version 3 <http://www.gnu.org/licenses/>

// -----------------------------------------------------------------------------
#define SPON_THIS_FILE SPON\Recognise\groupID
//
// Description:
//   Finds out the actual ID number of a person within his group as assigned by
//   the game and used in the squad leader's command menu, not just the order
//   within the units of his group (this order can change due to players joining
//   and leaving the game, deaths or promotions).
//
//   Inspired by the OFP function, squadNumber.sqf, by General Barron
//   (http://www.ofpec.com/ed_depot/index_new.php?action=details&id=139&page=1&game=OFP&type=fu&cat=xyz)
//
// Parameters:
//   0: _man - Man to check [Object: "Man"]
//
// Returns:
//   Number of person in his group [Number: 1+]
//
// -----------------------------------------------------------------------------

#include "macros\macros.inc.sqf"

#define ASCII_SPACE 32

// -----------------------------------------------------------------------------

SPON_GET_PARAMS_1(_man);

private ["_varName", "_manLabel", "_number", "_labelArray", "_groupLabelLen"];

// Save and remove the person's varName, so that format doesn't just show that.
_varName = vehicleVarName _man;
_man setVehicleVarName "";

_labelArray = toArray (str _man);
_groupLabelLen = count toArray (str (group _man));
_number = [];

// Read in all digits after the group name and colon in the full player label.
// Format of player label is "<groupName>:<groupNumber> <playerName>"
for "_i" from (_groupLabelLen + 1) to ((count _labelArray) - 1) do
{
	if ((_labelArray select _i) == ASCII_SPACE) exitWith {};
	SPON_PUSH(_number,_labelArray select _i);
};

// Go back to the original varName.
_man setVehicleVarName _varName;

parseNumber (toString _number); // Return.
